import logging
import os
from typing import Dict, Tuple

import yaml
from cerberus import Validator


def validate_resume(
    yaml_resume: Dict[str, Dict], schema_filename: str
) -> Tuple[str, bool]:
    """
    [Validates a certain resume ]

    Args:
        context_location (str): [description]
        yaml_resume (Dict[str, Dict]): [description]

    Raises:
        ValueError: [description]

    Returns:
        [type]: [description]
    """
    with open(schema_filename, "r") as file:
        yaml_schema = yaml.load(file, Loader=yaml.FullLoader)

    validator = Validator(yaml_schema)
    validator.allow_unknown = True  # type: ignore

    if validator.validate(yaml_resume):  # type: ignore
        info_message = (
            f"Resume in location {yaml_resume} validates to the resume_schema"
        )
        validation_flag = True
        return info_message, validation_flag
    else:
        warning_message = f"The resume in location {yaml_resume} does not validate to the resume_schema\n\tErrors: {validation_error}".format(validation_error=validator.errors)  # type: ignore
        validation_flag = False
        return warning_message, validation_flag



import json
import logging
import os
from pathlib import Path
from typing import Tuple
import uuid

import validators  # To validate that a certain string is a url
from dotenv import load_dotenv  # Use of environment variables
from validators.utils import ValidationFailure


def is_url_image(url: str) -> Tuple[str, bool]:
    """
    [Check if an url is an image]

    Args:
        url (str): [The url to check]

    Returns:
        str: [The content type of the url image]
        bool: [A boolean that indicates if the url is an image]
    """
    load_dotenv()
    image_formats = json.loads(str(os.getenv("IMAGE_FORMATS")))
    if not is_string_an_url(url):
        return "Not a url", False
    else:
        r = requests.head(url, timeout=1)  # type: ignore
        if r.headers["content-type"] in image_formats:
            return str(r.headers["content-type"]), True
        return str(r.headers), False


def is_string_an_url(url_string: str) -> bool:
    """
    [Check if a string is a valid url]

    Args:
        url_string (str): [the string to validate]

    Returns:
        bool: [a boolean indicating if the string is a valid url]
    """
    result = validators.url(url_string)  # type: ignore

    if isinstance(result, ValidationFailure):
        return False
    else:
        return True


def store_image_to_temp(image_folder: str, url: str) -> str:
    """
    [Store an image to a temporary folder]

    Args:
        image_folder (str): [The temporary folder to save the image to]
        url (str): [The url to fetch the image from]

    Returns:
        str: [The location of the image]
    """
    logging.info(
        f"Downloading badge image from location {url} to folder {image_folder}"
    )
    header, valid_url = is_url_image(url)
    if valid_url:
        img_data = requests.get(url, timeout=1).content  # type: ignore
        with open(Path(image_folder, str(uuid.uuid4())), "wb") as file:
            file.write(img_data)
        return file.name
    else:
        raise ValueError(
            f"The url '{url}' does not contain a valid image url. (Header type: {header})"
        )



import logging
import os
import tempfile
from typing import Dict, List

import magic
import requests  # To request the images listed in the badges
import yaml  # Retrieve the context
from docx.shared import Mm  # Set the height of the inlined image
from docxtpl import DocxTemplate, InlineImage  # Create Microsoft Word template
from jinja2 import Environment  # To pass custom filters to template

from resume_as_code.util import store_image_to_temp
from resume_as_code.validation import validate_resume


def fetch_resume_template(template_location: str) -> DocxTemplate:
    """
    [Retrieve the resume template]

    Args:
        template_location (str): [the location where the resume template is stored]
    Raises:
        FileNotFoundError: [if the file with filename provided in template_location does not exist]
        ValueError: [if the file is of the wrong filetype]
    Returns:
        DocxTemplate: [the Docx-Jinja-template]
    """
    if not os.path.exists(template_location):
        raise FileNotFoundError(
            "There is no file with location {}".format(template_location)
        )

    if "Microsoft Word" not in magic.from_file(template_location):
        raise ValueError(
            "Wrong filetype provided for the template file with location {}".format(
                template_location
            )
        )

    return DocxTemplate(template_location)


def fetch_yaml_context(context_location: str, schema_filename: str) -> Dict[str, Dict]:
    """
    [Retrieve the yaml context from its file and return as dictionary]

    Args:
        context_location (str): [the location where the yaml-resume is stored]
        schema_filename (str): [the filename containing the yaml specification]
    Raises:
        FileNotFoundError: [if the file with filename provided in context_location does not exist]
        ValueError: [if the file with the filename provided in context_location does not adhere to the spec defined in $SCHEMA_FILENAME]
    Returns:
        Dict[str, Dict]: [a dictionairy of key-value pairs]
    """
    if not os.path.exists(context_location):
        raise FileNotFoundError(
            "There is no file with location {}".format(context_location)
        )

    with open(context_location, "r") as file:
        yaml_resume = yaml.load(file, Loader=yaml.FullLoader)

    msg, validation_flag = validate_resume(yaml_resume, schema_filename)
    if validation_flag:
        logging.info(msg.format(context_location=context_location))
        return yaml_resume
    else:
        raise ValueError(msg.format(context_location=context_location))


def render_and_save(
    template: DocxTemplate,
    context: Dict[str, Dict],
    jinja_env: Environment,
    target_location: str,
) -> int:
    """
    [Render the template with the provided context and store in the target location]

    Args:
        template (DocxTemplate): [the Jinja-parametrized template]
        context (Dict[str, Dict]): [the YAML-context retrieved from the resume]
        jinja_env (Environment): [the jinja2 Environment to be passed down to the template]
        target_location (str): [the location to store the file in]

    Returns:
        int: [an integer status code]
    """
    with tempfile.TemporaryDirectory() as image_folder:
        preprocess_context(context, template, image_folder)
    template.render(context=context, jinja_env=jinja_env)
    template.save(target_location)
    return 0


def preprocess_context(
    context: Dict[str, Dict], template: DocxTemplate, image_folder: str
) -> None:
    """
    [Preprocess the context. Inline the images.]

    Args:
        context (Dict[str, Dict]): [The dict containing the yaml resume-values]
        template (DocxTemplate): [The template to inline the images on]
        image_folder (str): [The name of the temporary directory to save the images in]
    """
    for cert in context["contact"]["certifications"]:
        if "badge_image" not in cert.keys():
            continue
        badge_location = cert["badge_image"]
        try:
            image_location = store_image_to_temp(image_folder, badge_location)
            logging.info(f"Inlining image from location {image_location}")
            cert["image"] = InlineImage(template, image_location, height=Mm(20))
        except ValueError as e:
            # In case of an invalid url, the image shouldn't be inlined. Log the error though.
            logging.warning(e)
        except requests.exceptions.ConnectionError as e:  # type: ignore
            logging.warning(f"The url in string {badge_location} does not exist")


def with_badge(certs: List[Dict[str, str]]) -> List[Dict[str, str]]:
    """
    [Filter those certifications with a badge]

    Args:
        certs (List[Dict[str, str]]): [The unfiltered list of certifications]

    Returns:
        List[Dict[str, str]]: [Only those certifications that have a badge_image linked to it]
    """
    badged_certs = [cert for cert in certs if "badge_image" in cert.keys()]
    return badged_certs


def without_badge(certs: List[Dict[str, str]]) -> List[Dict[str, str]]:
    """
    [Filter those certifications with a badge]

    Args:
        certs (List[Dict[str, str]]): [The unfiltered list of certifications]

    Returns:
        List[Dict[str, str]]: [Only those certifications that have a badge_image linked to it]
    """
    badged_certs = [cert for cert in certs if "badge_image" not in cert.keys()]
    return badged_certs


def generate_resume(
    template_location: str,
    context_location: str,
    target_location: str,
    schema_location: str,
) -> int:
    """
    [Go through the entire flow of generating the resume]

    Args:
        template_location (str): [the location where the template is stored]
        context_location (str): [the location where the context is stored]
        target_location (str): [the location to store the generated resume in]
        schema_location (str): [the location where the yaml specification is stored]

    Returns:
        int: [the status code to indicate succes/not]
    """
    template = fetch_resume_template(template_location)
    try:
        context = fetch_yaml_context(context_location, schema_location)
    except ValueError as _:
        logging.error(
            f"The context in '{context_location}' does not contain a valid resume. Please review the spec in {schema_location}"
        )
        exit(1)
    jinja_env = Environment()
    jinja_env.filters["with_badge"] = with_badge
    jinja_env.filters["without_badge"] = without_badge
    status_code = render_and_save(template, context, jinja_env, target_location)
    return status_code



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['resume_as_code']

package_data = \
{'': ['*']}

install_requires = \
['Cerberus>=1.3,<2.0',
 'Jinja2>=3,<4',
 'PyYAML>=6,<7',
 'docxtpl>=0.15.2,<0.16.0',
 'python-docx>=0.8.11,<0.9.0',
 'python-dotenv>=0.19.2,<0.20.0',
 'python-magic>=0.4.24,<0.5.0',
 'requests>=2.27.1,<3.0.0',
 'validators>=0.18.2,<0.19.0']

setup_kwargs = {
    'name': 'resume-as-code',
    'version': '0.0.2',
    'description': 'A tool for automatic resume generation based on Jinja-Word-templates and YAML-files',
    'long_description': None,
    'author': 'Tim Van Erum',
    'author_email': 'tim.vanerum@dataroots.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.0,<4.0.0',
}


setup(**setup_kwargs)
